// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:ducene/ducene.dart';
import 'package:test/test.dart';

void main() {
  group('A group of highlighter', () {
    Analyzer ws;
    Analyzer uniGram;
    Analyzer hl;
    String field = 'dummy';
    String text = 'AAA Keyword BBB';

    setUp(() {
      ws = new WhitespaceAnalyzer();
      uniGram = new NGramAnalyzer(1);
      hl = new HighlightUniGramAnalyzer();
    });

    test('highlight whitespace', () {
      BoolQuery q = new BoolQuery(Op.or).append(field, "Keyword", ws);
      String r = Highlighter.getSnippet(q, field, text, ws, separator: ' ');
      expect(r, "aaa <b>keyword</b> bbb");
    });

    test('highlight whitespace lower', () {
      BoolQuery q = new BoolQuery(Op.or).append(field, "keyword", ws);
      String r = Highlighter.getSnippet(q, field, text, ws, separator: ' ');
      expect(r, "aaa <b>keyword</b> bbb");
    });

    test('highlight 1 gram 1 char', () {
      BoolQuery q = new BoolQuery(Op.or).append(field, "K", uniGram);
      String r = Highlighter.getSnippet(q, field, text, uniGram);
      expect(r, "aaa <b>k</b>eyword bbb");
    });

    test('HighlightAnalyzer 1 gram 1 char', () {
      BoolQuery q = new BoolQuery(Op.or).append(field, "K", hl);
      String r = Highlighter.getSnippet(q, field, text, hl);
      expect(r, "AAA <b>K</b>eyword BBB");
    });

    test('getHighlightSnippet() 1 gram 1 char', () {
      BoolQuery q = new BoolQuery(Op.or).append(field, "K", uniGram);
      String r = Highlighter.getHighlightSnippet(q, field, text);
      expect(r, "AAA <b>K</b>eyword BBB");
    });

    test('highlight 1 gram 1 char lower', () {
      BoolQuery q = new BoolQuery(Op.or).append(field, "k", uniGram);
      String r = Highlighter.getSnippet(q, field, text, uniGram);
      expect(r, "aaa <b>k</b>eyword bbb");
    });

    test('HighlightAnalyzer lower', () {
      BoolQuery q = new BoolQuery(Op.or).append(field, "k", hl);
      String r = Highlighter.getSnippet(q, field, text, hl);
      expect(r, "AAA Keyword BBB");
    });

    test('getHighlightSnippet() lower', () {
      BoolQuery q = new BoolQuery(Op.or).append(field, "k", uniGram);
      String r = Highlighter.getHighlightSnippet(q, field, text);
      expect(r, "AAA <b>K</b>eyword BBB");
    });

    test('highlight 1 gram 2 char', () {
      BoolQuery q = new BoolQuery(Op.or).append(field, "Ke", uniGram);
      String r = Highlighter.getSnippet(q, field, text, uniGram);
      expect(r, "aaa <b>k</b><b>e</b>yword bbb");
    });

    test('HighlightAnalyzer 2 char', () {
      BoolQuery q = new BoolQuery(Op.or).append(field, "Ke", hl);
      String r = Highlighter.getSnippet(q, field, text, hl);
      expect(r, "AAA <b>K</b><b>e</b>yword BBB");
    });

    test('getHighlightSnippet() 2 char', () {
      BoolQuery q = new BoolQuery(Op.or).append(field, "Ke", uniGram);
      String r = Highlighter.getHighlightSnippet(q, field, text);
      expect(r, "AAA <b>K</b><b>e</b>yword BBB");
    });

  });

  group('A group of fieldHighlight', () {
    Analyzer ws;

    setUp(() {
      ws = new WhitespaceAnalyzer();
    });

    test('test getTerms()', () {
      BoolQuery q = new BoolQuery(Op.or)
      ..append("f1", "f1-1", ws)
      ..append("f1", "f1-2", ws)
      ..append("f2", "f2-1", ws);
      expect(FieldHighlight.getTerms(q, "f1"), ["f1-1", "f1-2"]);
    });

    test('test getOffsets()', () {
      List<String> terms
      = ["keyword", 'bb', "wo", "word",
      "a keyword bb", "a keyword bb"
      " keyword ", " keyword", "cc"];
      String text = "AAA keyword BBB CCC";
      expect(FieldHighlight.getOffsets(terms, text), [{2: 14}, {16: 18}]);
    });

    test('test getOffsets()', () {
      List<String> terms
      = ["aaa bbb", "bbb ccc"];
      String text = "AAA BBB CCC";
      expect(FieldHighlight.getOffsets(terms, text), [{0: 7}]);
    });

    test('test markTags()', () {
      String text = "0123456789";
      var r = FieldHighlight.markTags([{1: 3}, {5: 7}], text, '<b>', '</b>');
      expect(r, '0<b>12</b>34<b>56</b>789');
    });

    test('test getSnippet()', () {
      BoolQuery q = new BoolQuery(Op.or)
          .append("f1", "keyword", ws)
          .append("f1", "bbb", ws);
      String text = "AAA keyword BBB";
      expect(FieldHighlight.getSnippet(q, 'f1', text),
          'AAA <b>keyword</b> <b>BBB</b>');
    });
  });

}